package main

import (
	"fmt"
	"os"
)

/*
	this parses the command line arguements using alex flint's go-arg
*/

var argv args

type args struct {
	Checkout *CheckoutCmd `arg:"subcommand:checkout"                  help:"switch git branches"`
	Config   *ConfigCmd   `arg:"subcommand:config"                    help:"show your .config/forge/ settings"`
	Dirty    *DirtyCmd    `arg:"subcommand:dirty"                     help:"check if your git repos are dirty"`
	GitReset *EmptyCmd    `arg:"subcommand:hard-reset"                help:"hard reset your user git branches"`
	List     *FindCmd     `arg:"subcommand:list"                      help:"just show a table of the current state"`
	Patch    *PatchCmd    `arg:"subcommand:patch"                     help:"examine and make patch sets"`
	GitPull  *FindCmd     `arg:"subcommand:pull"                      help:"run 'git pull'"`
	GitFetch *FindCmd     `arg:"subcommand:fetch"                     help:"run 'git fetch master'"`
	Rescan   *EmptyCmd    `arg:"subcommand:rescan"                    help:"recreate the git protobuf repos.pb file"`
	Delete   *EmptyCmd    `arg:"subcommand:delete"                    help:"untrack a repo"`
	Commit   *EmptyCmd    `arg:"subcommand:commit"                    help:"smart 'git commit' (errors out if on wrong branch)"`
	Clean    *CleanCmd    `arg:"subcommand:clean"                     help:"clean out all local branches (safely)"`
	Examine  *ExamineCmd  `arg:"subcommand:examine"                   help:"examine branches"`
	URL      string       `arg:"--connect"                            help:"gowebd url"`
	All      bool         `arg:"--all"                                help:"git commit --all"`
	Show     string       `arg:"--show"                               help:"show a repo"`
	Bash     bool         `arg:"--bash"                               help:"generate bash completion"`
	BashAuto []string     `arg:"--auto-complete"                      help:"does the actual autocompletion"`
	Force    bool         `arg:"--force"                              help:"try to strong arm things"`
	Verbose  bool         `arg:"--verbose"                            help:"show more output"`
}

type EmptyCmd struct {
}

type testCmd string

type ExamineCmd struct {
	Fix    *EmptyCmd `arg:"subcommand:fix"           help:"try to auto fix branches"`
	Show   *EmptyCmd `arg:"subcommand:show"          help:"show a specific patch"`
	Submit string    `arg:"--submit"                 help:"name of patchset"`
}

type CleanCmd struct {
	Force *EmptyCmd      `arg:"subcommand:force"         help:"dangerously delete things that are not pushed upstream"`
	User  *EmptyCmd      `arg:"subcommand:user"          help:"clean the user branches"`
	Devel *CleanDevelCmd `arg:"subcommand:devel"         help:"clean and verify the devel branches"`
}

type CleanDevelCmd struct {
	Force bool `arg:"--force"                              help:"try to strong arm things"`
}

type PatchCmd struct {
	List   *EmptyCmd `arg:"subcommand:list"          help:"list available patches"`
	Show   *EmptyCmd `arg:"subcommand:show"          help:"show a specific patch"`
	Submit string    `arg:"--submit"                 help:"submit a new patchset with name"`
}

type ConfigAddCmd struct {
	Path        string `arg:"--path"                            help:"absolute path of the git repo"`
	GoPath      string `arg:"--gopath"                          help:"GO path of the git repo"`
	Directory   bool   `arg:"--directory"    default:"false"    help:"repo is a directory to match against"`
	ReadOnly    bool   `arg:"--readonly"     default:"false"    help:"repo is readonly"`
	Writable    bool   `arg:"--writable"     default:"false"    help:"repo is writable"`
	Favorite    bool   `arg:"--favorite"     default:"false"    help:"forge will always go-clone or git clone this"`
	Private     bool   `arg:"--private"      default:"false"    help:"repo can not be published"`
	Interesting bool   `arg:"--interesting"  default:"false"    help:"something you decided was cool"`
	DebName     string `arg:"--debname"                         help:"the name of the debian package (or rpm, etc)"`
	Master      string `arg:"--master"                          help:"the git 'master' or 'main' branch name"`
	Devel       string `arg:"--devel"                           help:"the git devel branch name"`
	User        string `arg:"--user"                            help:"the git user branch name"`
}

type ConfigCmd struct {
	Add      *ConfigAddCmd `arg:"subcommand:add"                      help:"add a config setting"`
	Fix      *EmptyCmd     `arg:"subcommand:fix"                      help:"fix .config/forge/ and/or repos.pb protobuf file"`
	List     *EmptyCmd     `arg:"subcommand:list"                     help:"list your config settings"`
	Delete   string        `arg:"--delete"                            help:"delete this repo"`
	Register string        `arg:"--register"                          help:"register your git URL (foo.com/mystuff) or (github.com/foo/bar)"`
}

type CheckoutCmd struct {
	User   *FindCmd `arg:"subcommand:user"                      help:"git checkout user"`
	Devel  *FindCmd `arg:"subcommand:devel"                     help:"git checkout devel"`
	Master *FindCmd `arg:"subcommand:master"                    help:"git checkout master"`
}

type DirtyCmd struct {
	Show bool `arg:"--show-files"                         help:"also list every dirty file"`
}

type FindCmd struct {
	Patches   *EmptyCmd `arg:"subcommand:patches"              help:"show repos that have patches"`
	All       bool      `arg:"--all"                           help:"select every repo (the default)"`
	Mine      bool      `arg:"--mine"                          help:"your repos as defined in the forge config"`
	Favorites bool      `arg:"--favorites"                     help:"your repos configured as favorites"`
	Private   bool      `arg:"--private"                       help:"your private repos from your .config/forge/"`
	Dirty     bool      `arg:"--dirty"                         help:"only use dirty git repos"`
	User      bool      `arg:"--user"                          help:"show repos on the user branch"`
	// ReadOnly  bool `arg:"--readonly"                      help:"include read-only repos"`
}

func (args) Version() string {
	return ARGNAME + " " + VERSION + "    Built on " + BUILDTIME
}

func (a args) Description() string {
	return `
forge -- in the spirit of things like sourceforge

	This supports GO projects so far.
		It will work from ~/go/src or where your go.work file is.
		Since I mostly use ~/go/src, that has been tested more.

Examples:
	forge                                        # opens the GUI
	forge list                                   # show every repo state
	forge dirty                                  # check for dirty git repos
	forge pull                                   # run 'git pull' in every repo
	forge checkout                               # switch git branches

	forge config add --private --path /opt/bob   # add a private repo /opt/bob
	forge config add --directory --writable \
		--gopath 'go.wit.com/user/bob'       # directory contains r/w repos
	forge config add --private --writeable \
		--gopath 'go.wit.com/user/bob/work'  # a GO repo that can not be published

	`
}

func (args) doBashHelpDebug() {
	fmt.Fprintln(os.Stderr, "")
	fmt.Fprintln(os.Stderr, "hello world")
	var more string
	p0 := "0" + argv.BashAuto[0]
	p1 := "1" + argv.BashAuto[1]
	p2 := "2" + argv.BashAuto[2]
	if len(argv.BashAuto[1]) >= 0 {
		more = "more"
	} else {
		more = "less"
	}
	p1a := fmt.Sprintf("1a.%s.%+v.\n", argv.BashAuto[1], len(argv.BashAuto[1]))
	fmt.Fprintln(os.Stderr, "pull something else", argv.BashAuto, len(argv.BashAuto), p0, p1, p2, p1a, "end", more)
	fmt.Fprintln(os.Stderr, "")
}
